//
//  SIWorkManager.m
//  Kompressor2
//
//  Created by Daniel Vollmer on 18.12.06.
//  Copyright 2006 [maven] heavy industries. All rights reserved.
//

#import "SIWorkManager.h"
#import "SIWorkThread.h"
#import "SIWorkUnit.h"
#include <sys/sysctl.h>

@implementation SIWorkManager

- (id)initWithThreadCount:(int)maxThreads
{
	if (self = [super init])
	{
		int i;

//		NSLog(@"init: %@", self);
		_workQueue = [[NSMutableArray alloc] init];
		_workLock = [[NSConditionLock alloc] initWithCondition:0];
		for (i = 0; i < maxThreads; i++)
		{
			SIWorkThread *newThread = [[SIWorkThread alloc] initWithQueue:_workQueue withLock:_workLock];
			
			[NSThread detachNewThreadSelector:@selector(workLoop:) toTarget:newThread withObject:nil]; /* do not pass self as it will be retained */
			[newThread release];
		}
	}
	return self;
}

- (id)init
{
	int num_cores = 1;
	int mib[2] = {CTL_HW, HW_NCPU};
	size_t len = sizeof num_cores;
	
	sysctl(mib, 2, &num_cores, &len, NULL, 0);
//	NSLog(@"Found %i core(s).", num_cores);
	return [self initWithThreadCount:num_cores];
}

- (void)dealloc
{
//	NSLog(@"dealloc: %@", self);
	[_workLock lock];
	[_workQueue removeAllObjects]; /* don't want any more work done */
	[_workLock unlockWithCondition:1]; /* trigger one thread to exit, it will retrigger for the others */
	[_workQueue release];
	[_workLock release];
	[super dealloc];
}

- (void)addWorkUnit:(SIWorkUnit *)workUnit isHighPriority:(BOOL)important
{
	[_workLock lock];
	if (important)
		[_workQueue insertObject:workUnit atIndex:0]; // put in front of queue
	else
		[_workQueue addObject:workUnit];
	[_workLock unlockWithCondition:[_workQueue count] > 0];
}

- (int)removePendingWorkFor:(id)object
{
	int removeCount = 0;
	
	[_workLock lock];
	if (!object)
	{
		removeCount = [_workQueue count];
		[_workQueue removeAllObjects];
	}
	else
	{
		int i;
		
		for (i = [_workQueue count] - 1; i >= 0; i--)
		{
			SIWorkUnit *wu = [_workQueue objectAtIndex:i];
			
			if ([wu isRelatedTo:object])
			{
//				NSLog(@"Removed %@ from pending work.", wu);
				[_workQueue removeObjectAtIndex:i];
				removeCount++;
			}
		}
	}
	[_workLock unlockWithCondition:[_workQueue count] > 0];
	return removeCount;
}

@end
