//
//  SIWorkThread.m
//  Kompressor2
//
//  Created by Daniel Vollmer on 18.12.06.
//  Copyright 2006 [maven] heavy industries. All rights reserved.
//

#import "SIWorkThread.h"
#import "SIWorkUnit.h"

@implementation SIWorkThread

- (id)initWithQueue:(NSMutableArray *)aWorkQueue withLock:(NSConditionLock *)aWorkLock;
{
	if (self = [super init])
	{
//		NSLog(@"init: %@", self);
		_workQueue = [aWorkQueue retain];
		_workLock = [aWorkLock retain];
	}
	return self;
}

- (void)workLoop:(id)sender
{
	NSAutoreleasePool *outerPool = [[NSAutoreleasePool alloc] init];
	BOOL shouldExit = NO;
	
	do
	{
		NSAutoreleasePool *innerPool = [[NSAutoreleasePool alloc] init];
		SIWorkUnit *workUnit = nil;
		unsigned num_items = 0;

		[_workLock lockWhenCondition:1]; /* wait for work */
		num_items = [_workQueue count];
		if (num_items > 0)
		{
			workUnit = [[_workQueue objectAtIndex:0] retain];
			[_workQueue removeObjectAtIndex:0];
		}
		/* we want to trigger if we are asked to exit (i.e. woken up without items), or if there are work items left to do */
		[_workLock unlockWithCondition:num_items != 1];
		if (workUnit)
		{ /* do work */
			[workUnit performWork];
			[workUnit release];
		}
		else
			shouldExit = YES;
		[innerPool release];
	} while (!shouldExit);
	[outerPool release];
}

- (void)dealloc
{
//	NSLog(@"dealloc: %@", self);
	/* As the detached threads retain their target (i.e. us), we only dealloc when our thread is done */
	[_workQueue release];
	[_workLock release];
	[super dealloc];
}

@end
